(function() {
  var File, Status, _,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('../lodash');

  File = require('./file');

  Status = (function() {
    function Status(raw, repo) {
      this.repo = repo;
      if (!_.isPromisedGit(this.repo)) {
        throw new Error('Invalid repository object');
      }
      this.parseRaw(raw);
    }

    Status.prototype.parseRaw = function(raw) {
      var lines, ref;
      lines = this.splitLines(raw);
      this.branch = this.parseBranch(lines);
      return ref = this.parseFiles(lines), this.staged = ref[0], this.unstaged = ref[1], this.untracked = ref[2], ref;
    };

    Status.prototype.splitLines = function(raw) {
      return raw.split(indexOf.call(raw, '\u0000') >= 0 ? '\u0000' : '\n');
    };

    Status.prototype.parseBranch = function(lines) {
      var branch, ref;
      branch = (ref = lines[0].substring(3)) != null ? ref : 'HEAD';
      lines.shift();
      return branch;
    };

    Status.prototype.parseFiles = function(lines) {
      var file, i, len, line, mode, path, staged, unstaged, untracked;
      staged = [];
      unstaged = [];
      untracked = [];
      for (i = 0, len = lines.length; i < len; i++) {
        line = lines[i];
        if (!(line !== '')) {
          continue;
        }
        mode = line.substring(0, 2);
        path = line.substring(3);
        file = new File(path, this.repo, mode);
        if (file.isStaged()) {
          staged.push(file);
        }
        if (file.isUnstaged()) {
          unstaged.push(file);
        }
        if (file.isUntracked()) {
          untracked.push(file);
        }
      }
      return [staged, unstaged, untracked];
    };

    return Status;

  })();

  module.exports = Status;

}).call(this);
